
# q-expansion of newform 4800.2.a.w, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4800
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [0], [1], [0], [-1], [0], [-3], [4], [-4], [-7], [6], [6], [-9], [6], [-2], [-10], [1], [3], [-14], [10], [8], [-18], [0], [-3], [6], [8], [-2], [15], [-12], [16], [-6], [10], [-12], [-14], [-17], [9], [-19], [6], [4], [-6], [11], [-16], [11], [-18], [-13], [-7], [-11], [14], [23], [-18], [6], [-11], [-16], [-16], [-2], [-16], [-4], [11], [4], [5], [-30], [13], [-10], [23], [-12], [-32], [-19], [2], [30], [-36], [-30], [3], [-15], [-13], [-32], [-10], [13], [30], [5], [26], [-6], [36], [15], [-23], [20], [0], [-22], [18], [-20], [24], [18], [-29], [-8], [41], [-18], [-34], [20], [37], [-5], [16], [6], [8], [-20], [5], [31], [-18], [-22], [-32], [-21], [0], [10], [-2], [-19], [17], [-24], [12], [-28], [-10], [30], [10], [22], [-48], [24], [28], [48], [3], [36], [-7], [26], [20], [28], [48], [47], [44], [-1], [-18], [-7], [48], [-38], [-7], [42], [41], [-34], [-14], [-8], [37], [32], [-4], [-4], [-43], [-12], [-29], [-38], [-20], [42], [-1], [-8], [19], [-30], [-20], [-8], [-8], [54], [-42], [28], [-31], [-18], [5], [-30], [-60], [-23], [38], [-31], [59], [-20], [25], [-12], [41], [-35], [41], [48], [11], [-6], [46], [18], [3], [-11], [57], [-24], [9], [-10], [35], [60], [-12], [46], [-2], [-14], [-18], [56], [10], [56], [2], [50], [4], [12], [51], [-40], [-64], [67], [55], [-66], [21], [-12], [46], [14], [-39], [-60], [42], [-56], [-45], [-41], [42], [68], [-22], [-23], [-24], [2], [-41], [-36], [-5], [8], [-41], [46], [-24], [-48], [34], [6], [18], [26], [24], [-29], [11], [-39], [54], [-38], [51], [52], [-65], [-6], [-66], [-48], [-8], [21], [68], [-4], [43], [32], [-54], [14], [-51], [20], [-26], [61], [-62], [20], [34], [-28], [20], [-38], [-35], [12], [-74], [-59], [29], [-55], [-42], [5], [35], [-20], [-66], [32], [12], [21], [41], [30], [25], [34], [16], [-18], [50], [48], [32], [-68], [-31], [-78], [-55], [66], [-52], [-28], [-50], [68], [-75], [38], [-68], [45], [48], [31], [-42], [14], [-44], [40], [-60], [-9], [-8], [-7], [68], [-52], [41], [40], [-76], [-73], [-2], [-35], [46], [90], [-11], [-19], [84], [-14], [3], [74], [-43], [68], [-31], [60], [-55], [14], [-63], [-88], [17], [50], [12], [55], [24], [10], [23], [-63], [50], [-24], [-7], [38], [-92], [-25], [-51], [66], [-90], [-32], [-44], [-36], [73], [-70], [-72], [60], [80], [23], [64], [-16], [55], [-84], [-29], [64], [48], [-32], [83], [-70], [-90], [-49], [44], [-7], [54], [64], [48], [38], [44], [-54], [-19], [-58], [-40], [40], [-31], [2], [10], [47], [30], [29], [-11], [66], [-45], [-96], [86], [14], [47], [-22], [-38], [-40], [-69], [-82], [-23], [-96], [-17], [-84], [-14], [-27], [-95], [-36], [-66], [-57], [2], [-14], [4], [57], [-38], [14], [22], [58], [-70], [-14], [71], [36]]
